# coding: utf-8

"""
    CityPay Payment API

     Welcome to the CityPay API, a robust HTTP API payment solution designed for seamless server-to-server  transactional processing. Our API facilitates a wide array of payment operations, catering to diverse business needs.  Whether you're integrating Internet payments, handling Mail Order/Telephone Order (MOTO) transactions, managing  Subscriptions with Recurring and Continuous Authority payments, or navigating the complexities of 3-D Secure  authentication, our API is equipped to support your requirements. Additionally, we offer functionalities for  Authorisation, Refunding, Pre-Authorisation, Cancellation/Voids, and Completion processing, alongside the capability  for tokenised payments.  ## Compliance and Security Overview <aside class=\"notice\">   Ensuring the security of payment transactions and compliance with industry standards is paramount. Our API is    designed with stringent security measures and compliance protocols to safeguard sensitive information and meet    the rigorous requirements of Visa, MasterCard, and the PCI Security Standards Council. </aside>  ### Key Compliance and Security Measures  * **TLS Encryption**: All data transmissions must utilise TLS version 1.2 or higher, employing [strong cryptography](#enabled-tls-ciphers). Our infrastructure strictly enforces this requirement to maintain the integrity and confidentiality of data in transit. We conduct regular scans and assessments of our TLS endpoints to identify and mitigate vulnerabilities. * **Data Storage Prohibitions**: Storing sensitive cardholder data (CHD), such as the card security code (CSC) or primary account number (PAN), is strictly prohibited. Our API is designed to minimize your exposure to sensitive data, thereby reducing your compliance burden. * **Data Masking**: For consumer protection and compliance, full card numbers must not be displayed on receipts or any customer-facing materials. Our API automatically masks PANs, displaying only the last four digits to facilitate safe receipt generation. * **Network Scans**: If your application is web-based, regular scans of your hosting environment are mandatory to identify and rectify potential vulnerabilities. This proactive measure is crucial for maintaining a secure and compliant online presence. * **PCI Compliance**: Adherence to PCI DSS standards is not optional; it's a requirement for operating securely and legally in the payments ecosystem. For detailed information on compliance requirements and resources, please visit the PCI Security Standards Council website [https://www.pcisecuritystandards.org/](https://www.pcisecuritystandards.org/). * **Request Validation**: Our API includes mechanisms to verify the legitimacy of each request, ensuring it pertains to a valid account and originates from a trusted source. We leverage remote IP address verification alongside sophisticated application firewall technologies to thwart a wide array of common security threats.  ## Getting Started Before integrating with the CityPay API, ensure your application and development practices align with the outlined compliance and security measures. This preparatory step is crucial for a smooth integration process and the long-term success of your payment processing operations.  For further details on API endpoints, request/response formats, and code examples, proceed to the subsequent sections of our documentation. Our aim is to provide you with all the necessary tools and information to integrate our payment processing capabilities seamlessly into your application.  Thank you for choosing CityPay API. We look forward to supporting your payment processing needs with our secure, compliant, and versatile API solution. 

    Contact: support@citypay.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool, StrictInt
from pydantic import Field
from typing_extensions import Annotated
from citypay.models.airline_segment import AirlineSegment
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class AirlineAdvice(BaseModel):
    """
    AirlineAdvice
    """ # noqa: E501
    carrier_name: Annotated[str, Field(strict=True, max_length=25)] = Field(description="The name of the airline carrier that generated the tickets for airline travel.")
    conjunction_ticket_indicator: Optional[StrictBool] = Field(default=None, description="true if a conjunction ticket (with additional coupons) was issued for an itinerary with more than four segments. Defaults to false. ")
    eticket_indicator: Optional[StrictBool] = Field(default=None, description="The Electronic Ticket Indicator, a code that indicates if an electronic ticket was issued.  Defaults to true.")
    no_air_segments: Optional[Annotated[int, Field(le=4, strict=True, ge=0)]] = Field(default=None, description="A value that indicates the number of air travel segments included on this ticket.  Valid entries include the numerals “0” through “4”. Required only if the transaction type is TKT or EXC. ")
    number_in_party: Optional[StrictInt] = Field(default=None, description="The number of people in the party.")
    original_ticket_no: Optional[Annotated[str, Field(strict=True, max_length=14)]] = Field(default=None, description="Required if transaction type is EXC.")
    passenger_name: Optional[Annotated[str, Field(strict=True, max_length=25)]] = Field(default=None, description="The name of the passenger when the traveller is not the card member that purchased the ticket. Required only if the transaction type is TKT or EXC.")
    segment1: AirlineSegment
    segment2: Optional[AirlineSegment] = None
    segment3: Optional[AirlineSegment] = None
    segment4: Optional[AirlineSegment] = None
    ticket_issue_city: Annotated[str, Field(strict=True, max_length=18)] = Field(description="The name of the city town or village where the transaction took place.")
    ticket_issue_date: date = Field(description="The date the ticket was issued in ISO Date format (yyyy-MM-dd).")
    ticket_issue_name: Annotated[str, Field(strict=True, max_length=26)] = Field(description="The name of the agency generating the ticket.")
    ticket_no: Annotated[str, Field(strict=True, max_length=14)] = Field(description="This must be a valid ticket number, i.e. numeric (the first 3 digits must represent the valid IATA plate carrier code). The final check digit should be validated prior to submission. On credit charges, this field should contain the number of the original ticket, and not of a replacement. ")
    transaction_type: Annotated[str, Field(min_length=3, strict=True, max_length=3)] = Field(description="This field contains the Transaction Type code assigned to this transaction. Valid codes include:   - `TKT` = Ticket Purchase   - `REF` = Refund   - `EXC` = Exchange Ticket   - `MSC` = Miscellaneous (non-Ticket Purchase- and non-Exchange Ticket-related transactions only). ")
    __properties: ClassVar[List[str]] = ["carrier_name", "conjunction_ticket_indicator", "eticket_indicator", "no_air_segments", "number_in_party", "original_ticket_no", "passenger_name", "segment1", "segment2", "segment3", "segment4", "ticket_issue_city", "ticket_issue_date", "ticket_issue_name", "ticket_no", "transaction_type"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AirlineAdvice from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of segment1
        if self.segment1:
            _dict['segment1'] = self.segment1.to_dict()
        # override the default output from pydantic by calling `to_dict()` of segment2
        if self.segment2:
            _dict['segment2'] = self.segment2.to_dict()
        # override the default output from pydantic by calling `to_dict()` of segment3
        if self.segment3:
            _dict['segment3'] = self.segment3.to_dict()
        # override the default output from pydantic by calling `to_dict()` of segment4
        if self.segment4:
            _dict['segment4'] = self.segment4.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of AirlineAdvice from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "carrier_name": obj.get("carrier_name"),
            "conjunction_ticket_indicator": obj.get("conjunction_ticket_indicator"),
            "eticket_indicator": obj.get("eticket_indicator"),
            "no_air_segments": obj.get("no_air_segments"),
            "number_in_party": obj.get("number_in_party"),
            "original_ticket_no": obj.get("original_ticket_no"),
            "passenger_name": obj.get("passenger_name"),
            "segment1": AirlineSegment.from_dict(obj.get("segment1")) if obj.get("segment1") is not None else None,
            "segment2": AirlineSegment.from_dict(obj.get("segment2")) if obj.get("segment2") is not None else None,
            "segment3": AirlineSegment.from_dict(obj.get("segment3")) if obj.get("segment3") is not None else None,
            "segment4": AirlineSegment.from_dict(obj.get("segment4")) if obj.get("segment4") is not None else None,
            "ticket_issue_city": obj.get("ticket_issue_city"),
            "ticket_issue_date": obj.get("ticket_issue_date"),
            "ticket_issue_name": obj.get("ticket_issue_name"),
            "ticket_no": obj.get("ticket_no"),
            "transaction_type": obj.get("transaction_type")
        })
        return _obj


