# coding: utf-8

"""
    CityPay Payment API

     Welcome to the CityPay API, a robust HTTP API payment solution designed for seamless server-to-server  transactional processing. Our API facilitates a wide array of payment operations, catering to diverse business needs.  Whether you're integrating Internet payments, handling Mail Order/Telephone Order (MOTO) transactions, managing  Subscriptions with Recurring and Continuous Authority payments, or navigating the complexities of 3-D Secure  authentication, our API is equipped to support your requirements. Additionally, we offer functionalities for  Authorisation, Refunding, Pre-Authorisation, Cancellation/Voids, and Completion processing, alongside the capability  for tokenised payments.  ## Compliance and Security Overview <aside class=\"notice\">   Ensuring the security of payment transactions and compliance with industry standards is paramount. Our API is    designed with stringent security measures and compliance protocols to safeguard sensitive information and meet    the rigorous requirements of Visa, MasterCard, and the PCI Security Standards Council. </aside>  ### Key Compliance and Security Measures  * **TLS Encryption**: All data transmissions must utilise TLS version 1.2 or higher, employing [strong cryptography](#enabled-tls-ciphers). Our infrastructure strictly enforces this requirement to maintain the integrity and confidentiality of data in transit. We conduct regular scans and assessments of our TLS endpoints to identify and mitigate vulnerabilities. * **Data Storage Prohibitions**: Storing sensitive cardholder data (CHD), such as the card security code (CSC) or primary account number (PAN), is strictly prohibited. Our API is designed to minimize your exposure to sensitive data, thereby reducing your compliance burden. * **Data Masking**: For consumer protection and compliance, full card numbers must not be displayed on receipts or any customer-facing materials. Our API automatically masks PANs, displaying only the last four digits to facilitate safe receipt generation. * **Network Scans**: If your application is web-based, regular scans of your hosting environment are mandatory to identify and rectify potential vulnerabilities. This proactive measure is crucial for maintaining a secure and compliant online presence. * **PCI Compliance**: Adherence to PCI DSS standards is not optional; it's a requirement for operating securely and legally in the payments ecosystem. For detailed information on compliance requirements and resources, please visit the PCI Security Standards Council website [https://www.pcisecuritystandards.org/](https://www.pcisecuritystandards.org/). * **Request Validation**: Our API includes mechanisms to verify the legitimacy of each request, ensuring it pertains to a valid account and originates from a trusted source. We leverage remote IP address verification alongside sophisticated application firewall technologies to thwart a wide array of common security threats.  ## Getting Started Before integrating with the CityPay API, ensure your application and development practices align with the outlined compliance and security measures. This preparatory step is crucial for a smooth integration process and the long-term success of your payment processing operations.  For further details on API endpoints, request/response formats, and code examples, proceed to the subsequent sections of our documentation. Our aim is to provide you with all the necessary tools and information to integrate our payment processing capabilities seamlessly into your application.  Thank you for choosing CityPay API. We look forward to supporting your payment processing needs with our secure, compliant, and versatile API solution. 

    Contact: support@citypay.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from pydantic import Field
from typing_extensions import Annotated
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class ContactDetails(BaseModel):
    """
    ContactDetails
    """ # noqa: E501
    address1: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="The first line of the address for the card holder.")
    address2: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="The second line of the address for the card holder.")
    address3: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="The third line of the address for the card holder.")
    area: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="The area such as city, department, parish for the card holder.")
    company: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="The company name for the card holder if the contact is a corporate contact.")
    country: Optional[Annotated[str, Field(min_length=2, strict=True, max_length=2)]] = Field(default=None, description="The country code in ISO 3166 format. The country value may be used for fraud analysis and for   acceptance of the transaction. ")
    email: Optional[Annotated[str, Field(strict=True, max_length=254)]] = Field(default=None, description="An email address for the card holder which may be used for correspondence.")
    firstname: Optional[StrictStr] = Field(default=None, description="The first name  of the card holder.")
    lastname: Optional[StrictStr] = Field(default=None, description="The last name or surname of the card holder.")
    mobile_no: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="A mobile number for the card holder the mobile number is often required by delivery companies to ensure they are able to be in contact when required.")
    postcode: Optional[Annotated[str, Field(strict=True, max_length=16)]] = Field(default=None, description="The postcode or zip code of the address which may be used for fraud analysis.")
    telephone_no: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="A telephone number for the card holder.")
    title: Optional[StrictStr] = Field(default=None, description="A title for the card holder such as Mr, Mrs, Ms, M. Mme. etc.")
    __properties: ClassVar[List[str]] = ["address1", "address2", "address3", "area", "company", "country", "email", "firstname", "lastname", "mobile_no", "postcode", "telephone_no", "title"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ContactDetails from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ContactDetails from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "address1": obj.get("address1"),
            "address2": obj.get("address2"),
            "address3": obj.get("address3"),
            "area": obj.get("area"),
            "company": obj.get("company"),
            "country": obj.get("country"),
            "email": obj.get("email"),
            "firstname": obj.get("firstname"),
            "lastname": obj.get("lastname"),
            "mobile_no": obj.get("mobile_no"),
            "postcode": obj.get("postcode"),
            "telephone_no": obj.get("telephone_no"),
            "title": obj.get("title")
        })
        return _obj


