# coding: utf-8

"""
    CityPay Payment API

     Welcome to the CityPay API, a robust HTTP API payment solution designed for seamless server-to-server  transactional processing. Our API facilitates a wide array of payment operations, catering to diverse business needs.  Whether you're integrating Internet payments, handling Mail Order/Telephone Order (MOTO) transactions, managing  Subscriptions with Recurring and Continuous Authority payments, or navigating the complexities of 3-D Secure  authentication, our API is equipped to support your requirements. Additionally, we offer functionalities for  Authorisation, Refunding, Pre-Authorisation, Cancellation/Voids, and Completion processing, alongside the capability  for tokenised payments.  ## Compliance and Security Overview <aside class=\"notice\">   Ensuring the security of payment transactions and compliance with industry standards is paramount. Our API is    designed with stringent security measures and compliance protocols to safeguard sensitive information and meet    the rigorous requirements of Visa, MasterCard, and the PCI Security Standards Council. </aside>  ### Key Compliance and Security Measures  * **TLS Encryption**: All data transmissions must utilise TLS version 1.2 or higher, employing [strong cryptography](#enabled-tls-ciphers). Our infrastructure strictly enforces this requirement to maintain the integrity and confidentiality of data in transit. We conduct regular scans and assessments of our TLS endpoints to identify and mitigate vulnerabilities. * **Data Storage Prohibitions**: Storing sensitive cardholder data (CHD), such as the card security code (CSC) or primary account number (PAN), is strictly prohibited. Our API is designed to minimize your exposure to sensitive data, thereby reducing your compliance burden. * **Data Masking**: For consumer protection and compliance, full card numbers must not be displayed on receipts or any customer-facing materials. Our API automatically masks PANs, displaying only the last four digits to facilitate safe receipt generation. * **Network Scans**: If your application is web-based, regular scans of your hosting environment are mandatory to identify and rectify potential vulnerabilities. This proactive measure is crucial for maintaining a secure and compliant online presence. * **PCI Compliance**: Adherence to PCI DSS standards is not optional; it's a requirement for operating securely and legally in the payments ecosystem. For detailed information on compliance requirements and resources, please visit the PCI Security Standards Council website [https://www.pcisecuritystandards.org/](https://www.pcisecuritystandards.org/). * **Request Validation**: Our API includes mechanisms to verify the legitimacy of each request, ensuring it pertains to a valid account and originates from a trusted source. We leverage remote IP address verification alongside sophisticated application firewall technologies to thwart a wide array of common security threats.  ## Getting Started Before integrating with the CityPay API, ensure your application and development practices align with the outlined compliance and security measures. This preparatory step is crucial for a smooth integration process and the long-term success of your payment processing operations.  For further details on API endpoints, request/response formats, and code examples, proceed to the subsequent sections of our documentation. Our aim is to provide you with all the necessary tools and information to integrate our payment processing capabilities seamlessly into your application.  Thank you for choosing CityPay API. We look forward to supporting your payment processing needs with our secure, compliant, and versatile API solution. 

    Contact: support@citypay.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from pydantic import Field
from citypay.models.paylink_attachment_request import PaylinkAttachmentRequest
from citypay.models.paylink_email_notification_path import PaylinkEmailNotificationPath
from citypay.models.paylink_sms_notification_path import PaylinkSMSNotificationPath
from citypay.models.paylink_token_request_model import PaylinkTokenRequestModel
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class PaylinkBillPaymentTokenRequest(BaseModel):
    """
    PaylinkBillPaymentTokenRequest
    """ # noqa: E501
    addressee: Optional[StrictStr] = Field(default=None, description="Who the bill payment request intended for. This should be a readable name such as a person or company.")
    attachments: Optional[List[PaylinkAttachmentRequest]] = None
    descriptor: Optional[StrictStr] = Field(default=None, description="A descriptor for the bill payment used to describe what the payment request is for for instance \"Invoice\".  The descriptor can be used as descriptive text on emails or the payment page. For instance an invoice may have a button saying \"View Invoice\" or an email may say \"to pay your Invoice online\". ")
    due: Optional[date] = Field(default=None, description="A date that the invoice is due. This can be displayed on the payment page.")
    email_notification_path: Optional[PaylinkEmailNotificationPath] = None
    memo: Optional[StrictStr] = Field(default=None, description="A memo that can be added to the payment page and email to provide to the customer.")
    request: PaylinkTokenRequestModel
    sms_notification_path: Optional[PaylinkSMSNotificationPath] = None
    __properties: ClassVar[List[str]] = ["addressee", "attachments", "descriptor", "due", "email_notification_path", "memo", "request", "sms_notification_path"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PaylinkBillPaymentTokenRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in attachments (list)
        _items = []
        if self.attachments:
            for _item in self.attachments:
                if _item:
                    _items.append(_item.to_dict())
            _dict['attachments'] = _items
        # override the default output from pydantic by calling `to_dict()` of email_notification_path
        if self.email_notification_path:
            _dict['email_notification_path'] = self.email_notification_path.to_dict()
        # override the default output from pydantic by calling `to_dict()` of request
        if self.request:
            _dict['request'] = self.request.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sms_notification_path
        if self.sms_notification_path:
            _dict['sms_notification_path'] = self.sms_notification_path.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of PaylinkBillPaymentTokenRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "addressee": obj.get("addressee"),
            "attachments": [PaylinkAttachmentRequest.from_dict(_item) for _item in obj.get("attachments")] if obj.get("attachments") is not None else None,
            "descriptor": obj.get("descriptor"),
            "due": obj.get("due"),
            "email_notification_path": PaylinkEmailNotificationPath.from_dict(obj.get("email_notification_path")) if obj.get("email_notification_path") is not None else None,
            "memo": obj.get("memo"),
            "request": PaylinkTokenRequestModel.from_dict(obj.get("request")) if obj.get("request") is not None else None,
            "sms_notification_path": PaylinkSMSNotificationPath.from_dict(obj.get("sms_notification_path")) if obj.get("sms_notification_path") is not None else None
        })
        return _obj


