# coding: utf-8

"""
    CityPay Payment API

     Welcome to the CityPay API, a robust HTTP API payment solution designed for seamless server-to-server  transactional processing. Our API facilitates a wide array of payment operations, catering to diverse business needs.  Whether you're integrating Internet payments, handling Mail Order/Telephone Order (MOTO) transactions, managing  Subscriptions with Recurring and Continuous Authority payments, or navigating the complexities of 3-D Secure  authentication, our API is equipped to support your requirements. Additionally, we offer functionalities for  Authorisation, Refunding, Pre-Authorisation, Cancellation/Voids, and Completion processing, alongside the capability  for tokenised payments.  ## Compliance and Security Overview <aside class=\"notice\">   Ensuring the security of payment transactions and compliance with industry standards is paramount. Our API is    designed with stringent security measures and compliance protocols to safeguard sensitive information and meet    the rigorous requirements of Visa, MasterCard, and the PCI Security Standards Council. </aside>  ### Key Compliance and Security Measures  * **TLS Encryption**: All data transmissions must utilise TLS version 1.2 or higher, employing [strong cryptography](#enabled-tls-ciphers). Our infrastructure strictly enforces this requirement to maintain the integrity and confidentiality of data in transit. We conduct regular scans and assessments of our TLS endpoints to identify and mitigate vulnerabilities. * **Data Storage Prohibitions**: Storing sensitive cardholder data (CHD), such as the card security code (CSC) or primary account number (PAN), is strictly prohibited. Our API is designed to minimize your exposure to sensitive data, thereby reducing your compliance burden. * **Data Masking**: For consumer protection and compliance, full card numbers must not be displayed on receipts or any customer-facing materials. Our API automatically masks PANs, displaying only the last four digits to facilitate safe receipt generation. * **Network Scans**: If your application is web-based, regular scans of your hosting environment are mandatory to identify and rectify potential vulnerabilities. This proactive measure is crucial for maintaining a secure and compliant online presence. * **PCI Compliance**: Adherence to PCI DSS standards is not optional; it's a requirement for operating securely and legally in the payments ecosystem. For detailed information on compliance requirements and resources, please visit the PCI Security Standards Council website [https://www.pcisecuritystandards.org/](https://www.pcisecuritystandards.org/). * **Request Validation**: Our API includes mechanisms to verify the legitimacy of each request, ensuring it pertains to a valid account and originates from a trusted source. We leverage remote IP address verification alongside sophisticated application firewall technologies to thwart a wide array of common security threats.  ## Getting Started Before integrating with the CityPay API, ensure your application and development practices align with the outlined compliance and security measures. This preparatory step is crucial for a smooth integration process and the long-term success of your payment processing operations.  For further details on API endpoints, request/response formats, and code examples, proceed to the subsequent sections of our documentation. Our aim is to provide you with all the necessary tools and information to integrate our payment processing capabilities seamlessly into your application.  Thank you for choosing CityPay API. We look forward to supporting your payment processing needs with our secure, compliant, and versatile API solution. 

    Contact: support@citypay.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime as DateTime
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool, StrictInt, StrictStr
from pydantic import Field
from typing_extensions import Annotated
from citypay.models.paylink_state_event import PaylinkStateEvent
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class PaylinkTokenStatus(BaseModel):
    """
    PaylinkTokenStatus
    """ # noqa: E501
    amount_paid: Optional[StrictInt] = Field(default=None, description="the amount that has been paid against the session.")
    auth_code: Optional[StrictStr] = Field(default=None, description="an authorisation code if the transaction was processed and isPaid is true.")
    card: Optional[StrictStr] = Field(default=None, description="a description of the card that was used for payment if paid.")
    created: Optional[DateTime] = Field(default=None, description="the date and time that the session was created.")
    datetime: Optional[DateTime] = Field(default=None, description="the date and time of the current status.")
    identifier: Optional[Annotated[str, Field(min_length=4, strict=True, max_length=50)]] = Field(default=None, description="the merchant identifier, to help identifying the token.")
    is_attachment: Optional[StrictBool] = Field(default=None, description="true if an attachment exists.")
    is_cancelled: Optional[StrictBool] = Field(default=None, description="true if the session was cancelled either by the user or by a system request.")
    is_closed: Optional[StrictBool] = Field(default=None, description="true if the token has been closed.")
    is_customer_receipt_email_sent: Optional[StrictBool] = Field(default=None, description="true if a customer receipt has been sent.")
    is_email_sent: Optional[StrictBool] = Field(default=None, description="true if an email was sent.")
    is_expired: Optional[StrictBool] = Field(default=None, description="true if the session has expired.")
    is_form_viewed: Optional[StrictBool] = Field(default=None, description="true if the form was ever displayed to the addressee.")
    is_merchant_notification_email_sent: Optional[StrictBool] = Field(default=None, description="true if a merchant notification receipt was sent.")
    is_open_for_payment: Optional[StrictBool] = Field(default=None, description="true if the session is still open for payment or false if it has been closed.")
    is_paid: Optional[StrictBool] = Field(default=None, description="whether the session has been paid and therefore can be considered as complete.")
    is_payment_attempted: Optional[StrictBool] = Field(default=None, description="true if payment has been attempted.")
    is_postback_ok: Optional[StrictBool] = Field(default=None, description="true if a post back was executed successfully.")
    is_request_challenged: Optional[StrictBool] = Field(default=None, description="true if the request has been challenged using 3-D Secure.")
    is_sms_sent: Optional[StrictBool] = Field(default=None, description="true if an SMS was sent.")
    is_validated: Optional[StrictBool] = Field(default=None, description="whether the token generation was successfully validated.")
    last_event_date_time: Optional[DateTime] = Field(default=None, description="the date and time that the session last had an event actioned against it.")
    last_payment_result: Optional[StrictStr] = Field(default=None, description="the result of the last payment if one exists.")
    mid: Optional[StrictInt] = Field(default=None, description="identifies the merchant account.")
    payment_attempts_count: Optional[StrictInt] = Field(default=None, description="the number of attempts made to pay.")
    state_history: Optional[List[PaylinkStateEvent]] = None
    token: Optional[StrictStr] = Field(default=None, description="the token value which uniquely identifies the session.")
    trans_no: Optional[StrictInt] = Field(default=None, description="a transaction number if the transacstion was processed and isPaid is true.")
    __properties: ClassVar[List[str]] = ["amount_paid", "auth_code", "card", "created", "datetime", "identifier", "is_attachment", "is_cancelled", "is_closed", "is_customer_receipt_email_sent", "is_email_sent", "is_expired", "is_form_viewed", "is_merchant_notification_email_sent", "is_open_for_payment", "is_paid", "is_payment_attempted", "is_postback_ok", "is_request_challenged", "is_sms_sent", "is_validated", "last_event_date_time", "last_payment_result", "mid", "payment_attempts_count", "state_history", "token", "trans_no"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PaylinkTokenStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in state_history (list)
        _items = []
        if self.state_history:
            for _item in self.state_history:
                if _item:
                    _items.append(_item.to_dict())
            _dict['state_history'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of PaylinkTokenStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "amount_paid": obj.get("amount_paid"),
            "auth_code": obj.get("auth_code"),
            "card": obj.get("card"),
            "created": obj.get("created"),
            "datetime": obj.get("datetime"),
            "identifier": obj.get("identifier"),
            "is_attachment": obj.get("is_attachment"),
            "is_cancelled": obj.get("is_cancelled"),
            "is_closed": obj.get("is_closed"),
            "is_customer_receipt_email_sent": obj.get("is_customer_receipt_email_sent"),
            "is_email_sent": obj.get("is_email_sent"),
            "is_expired": obj.get("is_expired"),
            "is_form_viewed": obj.get("is_form_viewed"),
            "is_merchant_notification_email_sent": obj.get("is_merchant_notification_email_sent"),
            "is_open_for_payment": obj.get("is_open_for_payment"),
            "is_paid": obj.get("is_paid"),
            "is_payment_attempted": obj.get("is_payment_attempted"),
            "is_postback_ok": obj.get("is_postback_ok"),
            "is_request_challenged": obj.get("is_request_challenged"),
            "is_sms_sent": obj.get("is_sms_sent"),
            "is_validated": obj.get("is_validated"),
            "last_event_date_time": obj.get("last_event_date_time"),
            "last_payment_result": obj.get("last_payment_result"),
            "mid": obj.get("mid"),
            "payment_attempts_count": obj.get("payment_attempts_count"),
            "state_history": [PaylinkStateEvent.from_dict(_item) for _item in obj.get("state_history")] if obj.get("state_history") is not None else None,
            "token": obj.get("token"),
            "trans_no": obj.get("trans_no")
        })
        return _obj


